@@ -39,20 +39,14 @@ import io
 import importlib
 import itertools
 
-try:
+with contextlib.suppress(ImportError):
     import win32security
-except ImportError:
-    pass
 
-try:
+with contextlib.suppress(ImportError):
     import pwd
-except ImportError:
-    pass
 
-try:
+with contextlib.suppress(ImportError):
     import grp
-except ImportError:
-    pass
 
 from . import matchers
 from .py37compat import best_realpath, lru_cache
@@ -162,6 +156,8 @@ class Path(str):
     def __init__(self, other=''):
         if other is None:
             raise TypeError("Invalid initial value for path: None")
+        with contextlib.suppress(AttributeError):
+            self._validate()
 
     @classmethod
     @lru_cache
@@ -198,7 +194,7 @@ class Path(str):
 
     # The / operator joins Paths.
     def __div__(self, rel):
-        """ fp.__div__(rel) == fp / rel == fp.joinpath(rel)
+        """fp.__div__(rel) == fp / rel == fp.joinpath(rel)
 
         Join two path components, adding a separator character if
         needed.
@@ -212,7 +208,7 @@ class Path(str):
 
     # The / operator joins Paths the other way around
     def __rdiv__(self, rel):
-        """ fp.__rdiv__(rel) == rel / fp
+        """fp.__rdiv__(rel) == rel / fp
 
         Join two path components, adding a separator character if
         needed.
@@ -237,9 +233,9 @@ class Path(str):
 
     @classmethod
     def getcwd(cls):
-        """ Return the current working directory as a path object.
+        """Return the current working directory as a path object.
 
-        .. seealso:: :func:`os.getcwdu`
+        .. seealso:: :func:`os.getcwd`
         """
         return cls(os.getcwd())
 
@@ -247,40 +243,40 @@ class Path(str):
     # --- Operations on Path strings.
 
     def abspath(self):
-        """ .. seealso:: :func:`os.path.abspath` """
+        """.. seealso:: :func:`os.path.abspath`"""
         return self._next_class(self.module.abspath(self))
 
     def normcase(self):
-        """ .. seealso:: :func:`os.path.normcase` """
+        """.. seealso:: :func:`os.path.normcase`"""
         return self._next_class(self.module.normcase(self))
 
     def normpath(self):
-        """ .. seealso:: :func:`os.path.normpath` """
+        """.. seealso:: :func:`os.path.normpath`"""
         return self._next_class(self.module.normpath(self))
 
     def realpath(self):
-        """ .. seealso:: :func:`os.path.realpath` """
+        """.. seealso:: :func:`os.path.realpath`"""
         realpath = best_realpath(self.module)
         return self._next_class(realpath(self))
 
     def expanduser(self):
-        """ .. seealso:: :func:`os.path.expanduser` """
+        """.. seealso:: :func:`os.path.expanduser`"""
         return self._next_class(self.module.expanduser(self))
 
     def expandvars(self):
-        """ .. seealso:: :func:`os.path.expandvars` """
+        """.. seealso:: :func:`os.path.expandvars`"""
         return self._next_class(self.module.expandvars(self))
 
     def dirname(self):
-        """ .. seealso:: :attr:`parent`, :func:`os.path.dirname` """
+        """.. seealso:: :attr:`parent`, :func:`os.path.dirname`"""
         return self._next_class(self.module.dirname(self))
 
     def basename(self):
-        """ .. seealso:: :attr:`name`, :func:`os.path.basename` """
+        """.. seealso:: :attr:`name`, :func:`os.path.basename`"""
         return self._next_class(self.module.basename(self))
 
     def expand(self):
-        """ Clean up a filename by calling :meth:`expandvars()`,
+        """Clean up a filename by calling :meth:`expandvars()`,
         :meth:`expanduser()`, and :meth:`normpath()` on it.
 
         This is commonly everything needed to clean up a filename
@@ -290,7 +286,7 @@ class Path(str):
 
     @property
     def stem(self):
-        """ The same as :meth:`name`, but with one file extension stripped off.
+        """The same as :meth:`name`, but with one file extension stripped off.
 
         >>> Path('/home/guido/python.tar.gz').stem
         'python.tar'
@@ -300,12 +296,12 @@ class Path(str):
 
     @property
     def ext(self):
-        """ The file extension, for example ``'.py'``. """
+        """The file extension, for example ``'.py'``."""
         f, ext = self.module.splitext(self)
         return ext
 
     def with_suffix(self, suffix):
-        """ Return a new path with the file suffix changed (or added, if none)
+        """Return a new path with the file suffix changed (or added, if none)
 
         >>> Path('/home/guido/python.tar.gz').with_suffix(".foo")
         Path('/home/guido/python.tar.foo')
@@ -325,7 +321,7 @@ class Path(str):
 
     @property
     def drive(self):
-        """ The drive specifier, for example ``'C:'``.
+        """The drive specifier, for example ``'C:'``.
 
         This is always empty on systems that don't use drive specifiers.
         """
@@ -360,7 +356,7 @@ class Path(str):
     )
 
     def splitpath(self):
-        """ p.splitpath() -> Return ``(p.parent, p.name)``.
+        """Return two-tuple of ``.parent``, ``.name``.
 
         .. seealso:: :attr:`parent`, :attr:`name`, :func:`os.path.split`
         """
@@ -368,7 +364,7 @@ class Path(str):
         return self._next_class(parent), child
 
     def splitdrive(self):
-        """ p.splitdrive() -> Return ``(p.drive, <the rest of p>)``.
+        """Return two-tuple of ``.drive`` and rest without drive.
 
         Split the drive specifier from this path.  If there is
         no drive specifier, :samp:`{p.drive}` is empty, so the return value
@@ -380,7 +376,7 @@ class Path(str):
         return self._next_class(drive), rel
 
     def splitext(self):
-        """ p.splitext() -> Return ``(p.stripext(), p.ext)``.
+        """Return two-tuple of ``.stripext()`` and ``.ext``.
 
         Split the filename extension from this path and return
         the two parts.  Either part may be empty.
@@ -395,7 +391,7 @@ class Path(str):
         return self._next_class(filename), ext
 
     def stripext(self):
-        """ p.stripext() -> Remove one file extension from the path.
+        """Remove one file extension from the path.
 
         For example, ``Path('/home/guido/python.tar.gz').stripext()``
         returns ``Path('/home/guido/python.tar')``.
@@ -403,7 +399,6 @@ class Path(str):
         return self.splitext()[0]
 
     def splitunc(self):
-        """ .. seealso:: :func:`os.path.splitunc` """
         unc, rest = self.module.splitunc(self)
         return self._next_class(unc), rest
 
@@ -431,36 +426,45 @@ class Path(str):
         return first._next_class(first.module.join(first, *others))
 
     def splitall(self):
-        r""" Return a list of the path components in this path.
+        r"""Return a list of the path components in this path.
 
         The first item in the list will be a Path.  Its value will be
         either :data:`os.curdir`, :data:`os.pardir`, empty, or the root
         directory of this path (for example, ``'/'`` or ``'C:\\'``).  The
         other items in the list will be strings.
 
-        ``path.Path.joinpath(*result)`` will yield the original path.
+        ``Path.joinpath(*result)`` will yield the original path.
+
+        >>> Path('/foo/bar/baz').splitall()
+        [Path('/'), 'foo', 'bar', 'baz']
         """
-        parts = []
+        return list(self._parts())
+
+    def parts(self):
+        return tuple(self._parts())
+
+    def _parts(self):
+        return reversed(tuple(self._parts_iter()))
+
+    def _parts_iter(self):
         loc = self
         while loc != os.curdir and loc != os.pardir:
             prev = loc
             loc, child = prev.splitpath()
             if loc == prev:
                 break
-            parts.append(child)
-        parts.append(loc)
-        parts.reverse()
-        return parts
+            yield child
+        yield loc
 
     def relpath(self, start='.'):
-        """ Return this path as a relative path,
+        """Return this path as a relative path,
         based from `start`, which defaults to the current working directory.
         """
         cwd = self._next_class(start)
         return cwd.relpathto(self)
 
     def relpathto(self, dest):
-        """ Return a relative path from `self` to `dest`.
+        """Return a relative path from `self` to `dest`.
 
         If there is no relative path from `self` to `dest`, for example if
         they reside on different drives in Windows, then this returns
@@ -500,7 +504,7 @@ class Path(str):
     # --- Listing, searching, walking, and matching
 
     def listdir(self, match=None):
-        """ D.listdir() -> List of items in this directory.
+        """List of items in this directory.
 
         Use :meth:`files` or :meth:`dirs` instead if you want a listing
         of just files or just subdirectories.
@@ -516,7 +520,7 @@ class Path(str):
         return list(filter(match, (self / child for child in os.listdir(self))))
 
     def dirs(self, *args, **kwargs):
-        """ D.dirs() -> List of this directory's subdirectories.
+        """List of this directory's subdirectories.
 
         The elements of the list are Path objects.
         This does not walk recursively into subdirectories
@@ -527,7 +531,7 @@ class Path(str):
         return [p for p in self.listdir(*args, **kwargs) if p.isdir()]
 
     def files(self, *args, **kwargs):
-        """ D.files() -> List of the files in this directory.
+        """List of the files in self.
 
         The elements of the list are Path objects.
         This does not walk into subdirectories (see :meth:`walkfiles`).
@@ -538,7 +542,7 @@ class Path(str):
         return [p for p in self.listdir(*args, **kwargs) if p.isfile()]
 
     def walk(self, match=None, errors='strict'):
-        """ D.walk() -> iterator over files and subdirs, recursively.
+        """Iterator over files and subdirs, recursively.
 
         The iterator yields Path objects naming each child item of
         this directory and its descendants.  This requires that
@@ -554,29 +558,13 @@ class Path(str):
         `errors` may also be an arbitrary callable taking a msg parameter.
         """
 
-        class Handlers:
-            def strict(msg):
-                raise
-
-            def warn(msg):
-                warnings.warn(msg, TreeWalkWarning)
-
-            def ignore(msg):
-                pass
-
-        if not callable(errors) and errors not in vars(Handlers):
-            raise ValueError("invalid errors parameter")
-        errors = vars(Handlers).get(errors, errors)
-
+        errors = Handlers._resolve(errors)
         match = matchers.load(match)
 
         try:
             childList = self.listdir()
-        except Exception:
-            exc = sys.exc_info()[1]
-            tmpl = "Unable to list directory '%(self)s': %(exc)s"
-            msg = tmpl % locals()
-            errors(msg)
+        except Exception as exc:
+            errors(f"Unable to list directory '{self}': {exc}")
             return
 
         for child in childList:
@@ -586,37 +574,31 @@ class Path(str):
             traverse = traverse or child.isdir
             try:
                 do_traverse = traverse()
-            except Exception:
-                exc = sys.exc_info()[1]
-                tmpl = "Unable to access '%(child)s': %(exc)s"
-                msg = tmpl % locals()
-                errors(msg)
-                isdir = False
+            except Exception as exc:
+                errors(f"Unable to access '{child}': {exc}")
 
             if do_traverse:
                 for item in child.walk(errors=errors, match=match):
                     yield item
 
     def walkdirs(self, *args, **kwargs):
-        """ D.walkdirs() -> iterator over subdirs, recursively.
-        """
+        """Iterator over subdirs, recursively."""
         return (item for item in self.walk(*args, **kwargs) if item.isdir())
 
     def walkfiles(self, *args, **kwargs):
-        """ D.walkfiles() -> iterator over files in D, recursively.
-        """
+        """Iterator over files, recursively."""
         return (item for item in self.walk(*args, **kwargs) if item.isfile())
 
     def fnmatch(self, pattern, normcase=None):
-        """ Return ``True`` if `self.name` matches the given `pattern`.
+        """Return ``True`` if `self.name` matches the given `pattern`.
 
         `pattern` - A filename pattern with wildcards,
             for example ``'*.py'``. If the pattern contains a `normcase`
             attribute, it is applied to the name and path prior to comparison.
 
         `normcase` - (optional) A function used to normalize the pattern and
-            filename before matching. Defaults to :meth:`self.module`, which
-            defaults to :meth:`os.path.normcase`.
+            filename before matching. Defaults to normcase from
+            ``self.module``, :func:`os.path.normcase`.
 
         .. seealso:: :func:`fnmatch.fnmatch`
         """
@@ -627,7 +609,7 @@ class Path(str):
         return fnmatch.fnmatchcase(name, pattern)
 
     def glob(self, pattern):
-        """ Return a list of Path objects that match the pattern.
+        """Return a list of Path objects that match the pattern.
 
         `pattern` - a path relative to this directory, with wildcards.
 
@@ -644,7 +626,7 @@ class Path(str):
         return [cls(s) for s in glob.glob(self / pattern)]
 
     def iglob(self, pattern):
-        """ Return an iterator of Path objects that match the pattern.
+        """Return an iterator of Path objects that match the pattern.
 
         `pattern` - a path relative to this directory, with wildcards.
 
@@ -665,38 +647,38 @@ class Path(str):
     # --- Reading or writing an entire file at once.
 
     def open(self, *args, **kwargs):
-        """ Open this file and return a corresponding :class:`file` object.
+        """Open this file and return a corresponding file object.
 
         Keyword arguments work as in :func:`io.open`.  If the file cannot be
-        opened, an :class:`~exceptions.OSError` is raised.
+        opened, an :class:`OSError` is raised.
         """
         return io.open(self, *args, **kwargs)
 
     def bytes(self):
-        """ Open this file, read all bytes, return them as a string. """
+        """Open this file, read all bytes, return them as a string."""
         with self.open('rb') as f:
             return f.read()
 
     def chunks(self, size, *args, **kwargs):
-        """ Returns a generator yielding chunks of the file, so it can
-            be read piece by piece with a simple for loop.
+        """Returns a generator yielding chunks of the file, so it can
+         be read piece by piece with a simple for loop.
 
-           Any argument you pass after `size` will be passed to :meth:`open`.
+        Any argument you pass after `size` will be passed to :meth:`open`.
 
-           :example:
+        :example:
 
-               >>> hash = hashlib.md5()
-               >>> for chunk in Path("CHANGES.rst").chunks(8192, mode='rb'):
-               ...     hash.update(chunk)
+            >>> hash = hashlib.md5()
+            >>> for chunk in Path("CHANGES.rst").chunks(8192, mode='rb'):
+            ...     hash.update(chunk)
 
-            This will read the file by chunks of 8192 bytes.
+         This will read the file by chunks of 8192 bytes.
         """
         with self.open(*args, **kwargs) as f:
             for chunk in iter(lambda: f.read(size) or None, None):
                 yield chunk
 
     def write_bytes(self, bytes, append=False):
-        """ Open this file and write the given bytes to it.
+        """Open this file and write the given bytes to it.
 
         Default behavior is to overwrite any existing file.
         Call ``p.write_bytes(bytes, append=True)`` to append instead.
@@ -709,7 +691,7 @@ class Path(str):
             f.write(bytes)
 
     def read_text(self, encoding=None, errors=None):
-        r""" Open this file, read it in, return the content as a string.
+        r"""Open this file, read it in, return the content as a string.
 
         Optional parameters are passed to :meth:`open`.
 
@@ -724,7 +706,7 @@ class Path(str):
             return f.read()
 
     def text(self, encoding=None, errors='strict'):
-        """Legacy function to read text.
+        r"""Legacy function to read text.
 
         Converts all newline sequences to ``\n``.
         """
@@ -734,7 +716,7 @@ class Path(str):
     def write_text(
         self, text, encoding=None, errors='strict', linesep=os.linesep, append=False
     ):
-        r""" Write the given text to this file.
+        r"""Write the given text to this file.
 
         The default behavior is to overwrite any existing file;
         to append instead, use the `append=True` keyword argument.
@@ -807,7 +789,7 @@ class Path(str):
         self.write_bytes(text, append=append)
 
     def lines(self, encoding=None, errors='strict', retain=True):
-        r""" Open this file, read all lines, return them in a list.
+        r"""Open this file, read all lines, return them in a list.
 
         Optional arguments:
             `encoding` - The Unicode encoding (or character set) of
@@ -829,7 +811,7 @@ class Path(str):
     def write_lines(
         self, lines, encoding=None, errors='strict', linesep=os.linesep, append=False
     ):
-        r""" Write the given lines of text to this file.
+        r"""Write the given lines of text to this file.
 
         By default this overwrites any existing file at this path.
 
@@ -852,7 +834,7 @@ class Path(str):
                 default is os.linesep, which is platform-dependent
                 (``'\r\n'`` on Windows, ``'\n'`` on Unix, etc.).
                 Specify ``None`` to write the lines as-is, like
-                :meth:`file.writelines`.
+                ``.writelines`` on a file object.
 
         Use the keyword argument ``append=True`` to append lines to the
         file.  The default is to overwrite the file.
@@ -876,7 +858,7 @@ class Path(str):
                 f.write(line)
 
     def read_md5(self):
-        """ Calculate the md5 hash for this file.
+        """Calculate the md5 hash for this file.
 
         This reads through the entire file.
 
@@ -885,7 +867,7 @@ class Path(str):
         return self.read_hash('md5')
 
     def _hash(self, hash_name):
-        """ Returns a hash object for the file at the current path.
+        """Returns a hash object for the file at the current path.
 
         `hash_name` should be a hash algo name (such as ``'md5'``
         or ``'sha1'``) that's available in the :mod:`hashlib` module.
@@ -896,7 +878,7 @@ class Path(str):
         return m
 
     def read_hash(self, hash_name):
-        """ Calculate given hash for this file.
+        """Calculate given hash for this file.
 
         List of supported hashes can be obtained from :mod:`hashlib` package.
         This reads the entire file.
@@ -906,7 +888,7 @@ class Path(str):
         return self._hash(hash_name).digest()
 
     def read_hexhash(self, hash_name):
-        """ Calculate given hash for this file, returning hexdigest.
+        """Calculate given hash for this file, returning hexdigest.
 
         List of supported hashes can be obtained from :mod:`hashlib` package.
         This reads the entire file.
@@ -921,38 +903,38 @@ class Path(str):
     # bound. Playing it safe and wrapping them all in method calls.
 
     def isabs(self):
-        """ .. seealso:: :func:`os.path.isabs` """
+        """.. seealso:: :func:`os.path.isabs`"""
         return self.module.isabs(self)
 
     def exists(self):
-        """ .. seealso:: :func:`os.path.exists` """
+        """.. seealso:: :func:`os.path.exists`"""
         return self.module.exists(self)
 
     def isdir(self):
-        """ .. seealso:: :func:`os.path.isdir` """
+        """.. seealso:: :func:`os.path.isdir`"""
         return self.module.isdir(self)
 
     def isfile(self):
-        """ .. seealso:: :func:`os.path.isfile` """
+        """.. seealso:: :func:`os.path.isfile`"""
         return self.module.isfile(self)
 
     def islink(self):
-        """ .. seealso:: :func:`os.path.islink` """
+        """.. seealso:: :func:`os.path.islink`"""
         return self.module.islink(self)
 
     def ismount(self):
-        """ .. seealso:: :func:`os.path.ismount` """
+        """.. seealso:: :func:`os.path.ismount`"""
         return self.module.ismount(self)
 
     def samefile(self, other):
-        """ .. seealso:: :func:`os.path.samefile` """
+        """.. seealso:: :func:`os.path.samefile`"""
         if not hasattr(self.module, 'samefile'):
             other = Path(other).realpath().normpath().normcase()
             return self.realpath().normpath().normcase() == other
         return self.module.samefile(self, other)
 
     def getatime(self):
-        """ .. seealso:: :attr:`atime`, :func:`os.path.getatime` """
+        """.. seealso:: :attr:`atime`, :func:`os.path.getatime`"""
         return self.module.getatime(self)
 
     atime = property(
@@ -966,7 +948,7 @@ class Path(str):
     )
 
     def getmtime(self):
-        """ .. seealso:: :attr:`mtime`, :func:`os.path.getmtime` """
+        """.. seealso:: :attr:`mtime`, :func:`os.path.getmtime`"""
         return self.module.getmtime(self)
 
     mtime = property(
@@ -980,7 +962,7 @@ class Path(str):
     )
 
     def getctime(self):
-        """ .. seealso:: :attr:`ctime`, :func:`os.path.getctime` """
+        """.. seealso:: :attr:`ctime`, :func:`os.path.getctime`"""
         return self.module.getctime(self)
 
     ctime = property(
@@ -994,7 +976,7 @@ class Path(str):
     )
 
     def getsize(self):
-        """ .. seealso:: :attr:`size`, :func:`os.path.getsize` """
+        """.. seealso:: :attr:`size`, :func:`os.path.getsize`"""
         return self.module.getsize(self)
 
     size = property(
@@ -1010,7 +992,7 @@ class Path(str):
     if hasattr(os, 'access'):
 
         def access(self, mode):
-            """ Return ``True`` if current user has access to this path.
+            """Return ``True`` if current user has access to this path.
 
             mode - One of the constants :data:`os.F_OK`, :data:`os.R_OK`,
             :data:`os.W_OK`, :data:`os.X_OK`
@@ -1020,14 +1002,14 @@ class Path(str):
             return os.access(self, mode)
 
     def stat(self):
-        """ Perform a ``stat()`` system call on this path.
+        """Perform a ``stat()`` system call on this path.
 
         .. seealso:: :meth:`lstat`, :func:`os.stat`
         """
         return os.stat(self)
 
     def lstat(self):
-        """ Like :meth:`stat`, but do not follow symbolic links.
+        """Like :meth:`stat`, but do not follow symbolic links.
 
         .. seealso:: :meth:`stat`, :func:`os.lstat`
         """
@@ -1081,7 +1063,7 @@ class Path(str):
     if hasattr(os, 'statvfs'):
 
         def statvfs(self):
-            """ Perform a ``statvfs()`` system call on this path.
+            """Perform a ``statvfs()`` system call on this path.
 
             .. seealso:: :func:`os.statvfs`
             """
@@ -1090,14 +1072,14 @@ class Path(str):
     if hasattr(os, 'pathconf'):
 
         def pathconf(self, name):
-            """ .. seealso:: :func:`os.pathconf` """
+            """.. seealso:: :func:`os.pathconf`"""
             return os.pathconf(self, name)
 
     #
     # --- Modifying operations on files and directories
 
     def utime(self, times):
-        """ Set the access and modified times of this file.
+        """Set the access and modified times of this file.
 
         .. seealso:: :func:`os.utime`
         """
@@ -1135,12 +1117,12 @@ class Path(str):
         return self
 
     def rename(self, new):
-        """ .. seealso:: :func:`os.rename` """
+        """.. seealso:: :func:`os.rename`"""
         os.rename(self, new)
         return self._next_class(new)
 
     def renames(self, new):
-        """ .. seealso:: :func:`os.renames` """
+        """.. seealso:: :func:`os.renames`"""
         os.renames(self, new)
         return self._next_class(new)
 
@@ -1148,37 +1130,37 @@ class Path(str):
     # --- Create/delete operations on directories
 
     def mkdir(self, mode=0o777):
-        """ .. seealso:: :func:`os.mkdir` """
+        """.. seealso:: :func:`os.mkdir`"""
         os.mkdir(self, mode)
         return self
 
     def mkdir_p(self, mode=0o777):
-        """ Like :meth:`mkdir`, but does not raise an exception if the
-        directory already exists. """
+        """Like :meth:`mkdir`, but does not raise an exception if the
+        directory already exists."""
         with contextlib.suppress(FileExistsError):
             self.mkdir(mode)
         return self
 
     def makedirs(self, mode=0o777):
-        """ .. seealso:: :func:`os.makedirs` """
+        """.. seealso:: :func:`os.makedirs`"""
         os.makedirs(self, mode)
         return self
 
     def makedirs_p(self, mode=0o777):
-        """ Like :meth:`makedirs`, but does not raise an exception if the
-        directory already exists. """
+        """Like :meth:`makedirs`, but does not raise an exception if the
+        directory already exists."""
         with contextlib.suppress(FileExistsError):
             self.makedirs(mode)
         return self
 
     def rmdir(self):
-        """ .. seealso:: :func:`os.rmdir` """
+        """.. seealso:: :func:`os.rmdir`"""
         os.rmdir(self)
         return self
 
     def rmdir_p(self):
-        """ Like :meth:`rmdir`, but does not raise an exception if the
-        directory is not empty or does not exist. """
+        """Like :meth:`rmdir`, but does not raise an exception if the
+        directory is not empty or does not exist."""
         suppressed = FileNotFoundError, FileExistsError, DirectoryNotEmpty
         with contextlib.suppress(suppressed):
             with DirectoryNotEmpty.translate():
@@ -1186,13 +1168,13 @@ class Path(str):
         return self
 
     def removedirs(self):
-        """ .. seealso:: :func:`os.removedirs` """
+        """.. seealso:: :func:`os.removedirs`"""
         os.removedirs(self)
         return self
 
     def removedirs_p(self):
-        """ Like :meth:`removedirs`, but does not raise an exception if the
-        directory is not empty or does not exist. """
+        """Like :meth:`removedirs`, but does not raise an exception if the
+        directory is not empty or does not exist."""
         with contextlib.suppress(FileExistsError, DirectoryNotEmpty):
             with DirectoryNotEmpty.translate():
                 self.removedirs()
@@ -1201,7 +1183,7 @@ class Path(str):
     # --- Modifying operations on files
 
     def touch(self):
-        """ Set the access/modified times of this file to the current time.
+        """Set the access/modified times of this file to the current time.
         Create the file if it does not exist.
         """
         fd = os.open(self, os.O_WRONLY | os.O_CREAT, 0o666)
@@ -1210,25 +1192,25 @@ class Path(str):
         return self
 
     def remove(self):
-        """ .. seealso:: :func:`os.remove` """
+        """.. seealso:: :func:`os.remove`"""
         os.remove(self)
         return self
 
     def remove_p(self):
-        """ Like :meth:`remove`, but does not raise an exception if the
-        file does not exist. """
+        """Like :meth:`remove`, but does not raise an exception if the
+        file does not exist."""
         with contextlib.suppress(FileNotFoundError):
             self.unlink()
         return self
 
     def unlink(self):
-        """ .. seealso:: :func:`os.unlink` """
+        """.. seealso:: :func:`os.unlink`"""
         os.unlink(self)
         return self
 
     def unlink_p(self):
-        """ Like :meth:`unlink`, but does not raise an exception if the
-        file does not exist. """
+        """Like :meth:`unlink`, but does not raise an exception if the
+        file does not exist."""
         self.remove_p()
         return self
 
@@ -1237,7 +1219,7 @@ class Path(str):
     if hasattr(os, 'link'):
 
         def link(self, newpath):
-            """ Create a hard link at `newpath`, pointing to this file.
+            """Create a hard link at `newpath`, pointing to this file.
 
             .. seealso:: :func:`os.link`
             """
@@ -1247,7 +1229,7 @@ class Path(str):
     if hasattr(os, 'symlink'):
 
         def symlink(self, newlink=None):
-            """ Create a symbolic link at `newlink`, pointing here.
+            """Create a symbolic link at `newlink`, pointing here.
 
             If newlink is not supplied, the symbolic link will assume
             the name self.basename(), creating the link in the cwd.
@@ -1262,7 +1244,7 @@ class Path(str):
     if hasattr(os, 'readlink'):
 
         def readlink(self):
-            """ Return the path to which this symbolic link points.
+            """Return the path to which this symbolic link points.
 
             The result may be an absolute or a relative path.
 
@@ -1271,7 +1253,7 @@ class Path(str):
             return self._next_class(os.readlink(self))
 
         def readlinkabs(self):
-            """ Return the path to which this symbolic link points.
+            """Return the path to which this symbolic link points.
 
             The result is always an absolute path.
 
@@ -1298,14 +1280,14 @@ class Path(str):
     rmtree = shutil.rmtree
 
     def rmtree_p(self):
-        """ Like :meth:`rmtree`, but does not raise an exception if the
-        directory does not exist. """
+        """Like :meth:`rmtree`, but does not raise an exception if the
+        directory does not exist."""
         with contextlib.suppress(FileNotFoundError):
             self.rmtree()
         return self
 
     def chdir(self):
-        """ .. seealso:: :func:`os.chdir` """
+        """.. seealso:: :func:`os.chdir`"""
         os.chdir(self)
 
     cd = chdir
@@ -1316,7 +1298,7 @@ class Path(str):
         symlinks=False,
         *,
         copy_function=shutil.copy2,
-        ignore=lambda dir, contents: []
+        ignore=lambda dir, contents: [],
     ):
         """
         Copy entire contents of self to dst, overwriting existing
@@ -1347,7 +1329,10 @@ class Path(str):
                 target.symlink(dest)
             elif source.isdir():
                 source.merge_tree(
-                    dest, symlinks=symlinks, copy_function=copy_function, ignore=ignore,
+                    dest,
+                    symlinks=symlinks,
+                    copy_function=copy_function,
+                    ignore=ignore,
                 )
             else:
                 copy_function(source, dest)
@@ -1360,13 +1345,13 @@ class Path(str):
     if hasattr(os, 'chroot'):
 
         def chroot(self):
-            """ .. seealso:: :func:`os.chroot` """
+            """.. seealso:: :func:`os.chroot`"""
             os.chroot(self)
 
     if hasattr(os, 'startfile'):
 
         def startfile(self):
-            """ .. seealso:: :func:`os.startfile` """
+            """.. seealso:: :func:`os.startfile`"""
             os.startfile(self)
             return self
 
@@ -1415,11 +1400,8 @@ class Path(str):
         # move existing file to backup, create new file with same permissions
         # borrowed extensively from the fileinput module
         backup_fn = self + (backup_extension or os.extsep + 'bak')
-        try:
-            os.unlink(backup_fn)
-        except os.error:
-            pass
-        os.rename(self, backup_fn)
+        backup_fn.remove_p()
+        self.rename(backup_fn)
         readable = io.open(
             backup_fn,
             mode,
@@ -1431,8 +1413,7 @@ class Path(str):
         try:
             perm = os.fstat(readable.fileno()).st_mode
         except OSError:
-            writable = open(
-                self,
+            writable = self.open(
                 'w' + mode.replace('r', ''),
                 buffering=buffering,
                 encoding=encoding,
@@ -1452,31 +1433,22 @@ class Path(str):
                 errors=errors,
                 newline=newline,
             )
-            try:
-                if hasattr(os, 'chmod'):
-                    os.chmod(self, perm)
-            except OSError:
-                pass
+            with contextlib.suppress(OSError, AttributeError):
+                self.chmod(perm)
         try:
             yield readable, writable
         except Exception:
             # move backup back
             readable.close()
             writable.close()
-            try:
-                os.unlink(self)
-            except os.error:
-                pass
-            os.rename(backup_fn, self)
+            self.remove_p()
+            backup_fn.rename(self)
             raise
         else:
             readable.close()
             writable.close()
         finally:
-            try:
-                os.unlink(backup_fn)
-            except os.error:
-                pass
+            backup_fn.remove_p()
 
     @ClassProperty
     @classmethod
@@ -1532,6 +1504,35 @@ def only_newer(copy_func):
     return wrapper
 
 
+class ExtantPath(Path):
+    """
+    >>> ExtantPath('.')
+    ExtantPath('.')
+    >>> ExtantPath('does-not-exist')
+    Traceback (most recent call last):
+    OSError: does-not-exist does not exist.
+    """
+
+    def _validate(self):
+        if not self.exists():
+            raise OSError(f"{self} does not exist.")
+
+
+class ExtantFile(Path):
+    """
+    >>> ExtantFile('.')
+    Traceback (most recent call last):
+    FileNotFoundError: . does not exist as a file.
+    >>> ExtantFile('does-not-exist')
+    Traceback (most recent call last):
+    FileNotFoundError: does-not-exist does not exist as a file.
+    """
+
+    def _validate(self):
+        if not self.isfile():
+            raise FileNotFoundError(f"{self} does not exist as a file.")
+
+
 class SpecialResolver:
     class ResolverScope:
         def __init__(self, paths, scope):
@@ -1630,8 +1631,7 @@ class TempDir(Path):
         return self._next_class(self)
 
     def __exit__(self, exc_type, exc_value, traceback):
-        if not exc_value:
-            self.rmtree()
+        self.rmtree()
 
 
 def _multi_permission_mask(mode):
@@ -1708,3 +1708,20 @@ def _permission_mask(mode):
         '=': lambda mask, target: target & retain ^ mask,
     }
     return functools.partial(op_map[op], mask)
+
+
+class Handlers:
+    def strict(msg):
+        raise
+
+    def warn(msg):
+        warnings.warn(msg, TreeWalkWarning)
+
+    def ignore(msg):
+        pass
+
+    @classmethod
+    def _resolve(cls, param):
+        if not callable(param) and param not in vars(Handlers):
+            raise ValueError("invalid errors parameter")
+        return vars(cls).get(param, param)
